<?php

namespace WoocommercePplVendor\Jane\Component\JsonSchema\Guesser\JsonSchema;

use WoocommercePplVendor\Jane\Component\JsonSchema\Guesser\ChainGuesserAwareInterface;
use WoocommercePplVendor\Jane\Component\JsonSchema\Guesser\ChainGuesserAwareTrait;
use WoocommercePplVendor\Jane\Component\JsonSchema\Guesser\ClassGuesserInterface;
use WoocommercePplVendor\Jane\Component\JsonSchema\Guesser\Guess\ArrayType;
use WoocommercePplVendor\Jane\Component\JsonSchema\Guesser\Guess\MultipleType;
use WoocommercePplVendor\Jane\Component\JsonSchema\Guesser\Guess\Type;
use WoocommercePplVendor\Jane\Component\JsonSchema\Guesser\GuesserInterface;
use WoocommercePplVendor\Jane\Component\JsonSchema\Guesser\TypeGuesserInterface;
use WoocommercePplVendor\Jane\Component\JsonSchema\JsonSchema\Model\JsonSchema;
use WoocommercePplVendor\Jane\Component\JsonSchema\Registry\Registry;
use WoocommercePplVendor\Jane\Component\JsonSchema\Registry\Schema;
class ArrayGuesser implements GuesserInterface, TypeGuesserInterface, ChainGuesserAwareInterface, ClassGuesserInterface
{
    use ChainGuesserAwareTrait;
    /** @var array<string, int> */
    protected $refGuessLevel = [];
    /**
     * {@inheritdoc}
     */
    public function guessClass($object, string $name, string $reference, Registry $registry) : void
    {
        if (\is_a($object->getItems(), $this->getSchemaClass())) {
            $this->chainGuesser->guessClass($object->getItems(), $name . 'Item', $reference . '/items', $registry);
        }
    }
    /**
     * {@inheritdoc}
     */
    public function supportObject($object) : bool
    {
        return $object instanceof JsonSchema && 'array' === $object->getType();
    }
    /**
     * {@inheritdoc}
     */
    public function guessType($object, string $name, string $reference, Registry $registry) : Type
    {
        $this->refGuessLevel[$reference] = ($this->refGuessLevel[$reference] ?? 0) + 1;
        if ($this->refGuessLevel[$reference] > 20) {
            return new ArrayType($object, new Type($object, 'mixed'));
        }
        $items = $object->getItems();
        if (null === $items || \is_array($items) && 0 === \count($items)) {
            return new ArrayType($object, new Type($object, 'mixed'));
        }
        if (!\is_array($items)) {
            return new ArrayType($object, $this->chainGuesser->guessType($items, $name . 'Item', $reference . '/items', $registry));
        }
        $type = new MultipleType($object);
        foreach ($items as $key => $item) {
            $type->addType(new ArrayType($object, $this->chainGuesser->guessType($item, $name . 'Item', $reference . '/items/' . $key, $registry)));
        }
        return $type;
    }
    protected function getSchemaClass() : string
    {
        return Schema::class;
    }
}
