<?php
namespace WoocommercePpl\Admin\Category;

use WoocommercePpl\Admin\Assets\JsTemplate;
use WoocommercePpl\Model\Model\CategoryModel;
use WoocommercePpl\Model\Model\ShipmentMethodModel;
use WoocommercePpl\Serializer;
use WoocommercePpl\ShipmentMethod;

class CategoryPanel {

    public static function get_shipping()
    {
        $output = [];
        foreach (ShipmentMethod::methodsWithCod() as $k => $v) {
            $output[] = Serializer::getInstance()->normalize(new ShipmentMethodModel([
                "code" => $k,
                "title" => $v,
                "codAvailable" => ShipmentMethod::isMethodWithCod($k),
                "parcelRequired" => ShipmentMethod::isMethodWithParcel($k)
            ]), "array");
        }
        return $output;
    }

    public static function register()
    {
        //add_action('product_cat_edit_form_fields', [self::class, "render"], 10, 2);
        add_action('product_cat_add_form_fields', [self::class, "render"], 10);
        add_action('product_cat_edit_form_fields', [self::class, "render"], 10, 2);
        add_action('edited_product_cat', [self::class, "save"], 10, 2);
        add_action('created_product_cat', [self::class, "save"], 10, 2);
    }

    public static function save($term_id, $tt_id)
    {
        $model = new CategoryModel();
        $post = wp_parse_args($_POST, [
            "pplDisabledTransport" => [],
        ]);
        if (is_array($post['pplDisabledTransport'])) {
            $model->setPplDisabledTransport($post['pplDisabledTransport']);
        }
        $category = get_term($term_id);
        $category = Serializer::getInstance()->denormalize($model, \WP_Term::class, null, [ "category" => $category]);
    }

    public static function render($taxonomy = null)
    {
            $shipments = self::get_shipping();
            $categoryModel = Serializer::getInstance()->denormalize($taxonomy, CategoryModel::class);
            JsTemplate::scripts();
            if ($taxonomy === "product_cat")
            {
                ?>
                <div class="form-field">
                    <label >
                        Seznam zakázaných metod pro PPL
                    </label>
                    <div  id="woocommerce_ppl_tab">
                        <script type="text/javascript">
                            window.pplPlugin = window.pplPlugin || [];
                            pplPlugin.push([
                                "pplCategoryTab",
                                "woocommerce_ppl_tab",
                                {
                                    data: <?php echo json_encode( Serializer::getInstance()->normalize($categoryModel, "array") );?>,
                                    methods: <?php echo json_encode( $shipments );?>,
                                }
                            ])
                        </script>
                    </div>
                </div>
                <?php
            } else {
            ?>
            <tr class="form-field">
            <th >
                Seznam zakázaných metod pro PPL
            </th>
                <td  id="woocommerce_ppl_tab">
                    <script type="text/javascript">
                        window.pplPlugin = window.pplPlugin || [];
                        pplPlugin.push([
                            "pplCategoryTab",
                            "woocommerce_ppl_tab",
                            {
                                data: <?php echo json_encode( Serializer::getInstance()->normalize($categoryModel, "array") );?>,
                                methods: <?php echo json_encode( $shipments );?>,
                            }
                        ])
                    </script>
                </td>
            </tr>
            <?php
            }
    }
}