<?php
namespace WoocommercePpl\Admin;
defined("WPINC") or die();


use WoocommercePpl\Data\PackageData;
use WoocommercePpl\Data\ShipmentData;
use WpOrg\Requests\Exception;

class FileDownload {
    public static function rewrite_rule()
    {
        add_rewrite_rule("^ppl-download/([0-9]+)/?$", 'index.php?ppl_download=$matches[1]', "top");
    }

    public static function query_vars($vars)
    {
        $vars[] = 'ppl_download';
        $vars[] = 'ppl_reference';
        $vars[] = 'ppl_print';
        return $vars;
    }

    public static function request($vars) {
        if (isset($vars['ppl_download']) &&  wp_strip_all_tags($vars['ppl_download'])) {
            $vars['ppl_download'] = wp_strip_all_tags($vars['ppl_download']);
            if (!isset($vars["ppl_reference"]))
                $vars['ppl_reference'] = null;
            if (!isset($vars['ppl_print']))
                $vars['ppl_print'] = null;

        }
        return $vars;
    }

    public static function template_redirect()
    {
        global $wp_query;

        if (isset($wp_query->query_vars['ppl_download']) && $wp_query->query_vars['ppl_download']) {

            if (!current_user_can("manage_woocommerce")) {
                http_response_code(403);
                wp_die();
            }

            $shipmentId = $wp_query->query_vars['ppl_download'];

            try {
                if (is_numeric($shipmentId)) {
                    $downloadLabel = new CPLOperation();
                    $packageData = new PackageData($shipmentId);
                    if (!$packageData->get_id())
                        wp_die(__('File not found.'));
                    $shipmentId = $packageData->get_ppl_shipment_id();
                    $shipmentData = new ShipmentData($shipmentId);
                    $downloadLabel->getLabelContents($shipmentData->get_batch_id(), $shipmentData->get_reference_id(), $packageData->get_shipment_number(), $wp_query->query_vars['ppl_print']);
                } else if (preg_match('/^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}$/', $shipmentId)) {
                    $reference = @$wp_query->query_vars['ppl_reference'];
                    $finded = ShipmentData::read_shipments(["batch_label_group" => [$shipmentId]]);
                    if ($finded) {
                        $batch_id = $finded[0]->get_batch_id();
                        $operation = new CPLOperation();
                        $operation->getLabelContents($batch_id, $reference, null, $wp_query->query_vars['ppl_print']);
                    }
                }
            }
            catch (\Exception $exception)
            {
                wp_die(__('Problém se stažením, nepokoušíte se tisknout štítky starší než 31 dní?'));
            }
            wp_die(__('File not found.'));
        }
    }

    public static function register()
    {
        add_action('init', [self::class, 'rewrite_rule']);
        add_filter('query_vars', [self::class, 'query_vars']);
        add_filter("request", [self::class, "request"]);
        add_action('template_redirect', [self::class, 'template_redirect']);
    }


}