<?php
namespace WoocommercePpl\Admin\Order;
defined("WPINC") or die();


use Automattic\WooCommerce\Internal\DataStores\Orders\OrdersTableQuery;
use WoocommercePpl\Admin\CPLOperation;
use WoocommercePpl\Data\CollectionData;
use WoocommercePpl\Data\PPLData;
use WoocommercePpl\Data\ShipmentData;
use WoocommercePpl\Admin\Errors;
use WoocommercePpl\Model\Model\ShipmentModel;
use WoocommercePpl\Serializer;
use WoocommercePpl\ShipmentMethod;
use WoocommercePpl\Traits\ParcelDataModelTrait;
use WoocommercePpl\Validator\Validator;


class OrderFilter {

    use ParcelDataModelTrait;

    const NOTHING = "0";

    const PRINT = "1";


    public static function rerender()
    {
        if (!current_user_can("manage_woocommerce"))
            wp_die();
        ob_start();
        self::filters();
        wp_send_json_success([
            "html" => ob_get_clean()
        ]);
    }

    public static function filters() {
        static $rendered = false;


        $order_shipments = [] /* ShipmentData::read_label_groups()*/;

        $rendered = true;
        $filter = wp_parse_args($_REQUEST, ["ppl_filter" => self::NOTHING]);

        $filters =  [
            //["value" => self::NOTHING, "label" => "Tisk zásilek"],
        ];
        ob_start();
        menu_page_url(plugin_basename(realpath(__DIR__ . '/../Page/OptionPage.php')));

        $newCollectionUrl = ob_get_clean() . '#/collection/new';

        foreach ($order_shipments as $key => $value) {
            $filters[] = [
               "value" => $value, "label" => 'Tisk #' . ($key+1)
            ];
        }



        wc_get_template("ppl/admin/order-filter.php", [
            "filters" => $filters,
            "filter" => $filter["ppl_filter"],
            'newCollectionUrl' => $newCollectionUrl
        ]);

    }

    public static function query_args($query) {

        $filter = wp_parse_args($_GET, ["ppl_filter" => self::NOTHING]);
        if ($filter["ppl_filter"] !== self::NOTHING) {}
        {
            $query["ppl_filter"] = $filter["ppl_filter"];
        }
        return $query;
    }

    public static function query_clausule($sql, $item, $args)
    {
        global $wpdb;
        if (@$args["ppl_filter"]) {
            $order_shipments = ShipmentData::read_label_groups();
            foreach ($order_shipments as $datum) {
                if (@$args['ppl_filter'] === $datum) {

                    $q = $wpdb->prepare(" AND  {$wpdb->prefix}wc_orders.id in (select wc_order_id from {$wpdb->prefix}woocommerce_ppl_shipment where batch_label_group = %s ) ", $datum);
                    $sql["where"] .= $q;
                }
            }
        }

        if (isset($args["s"]) && $args['s'] && $args["search_filter"] === 'all') {
            $q =str_replace("likeMaker", "%", $wpdb->prepare(" OR {$wpdb->prefix}wc_orders.id in (select pplfilter_a.wc_order_id from {$wpdb->prefix}woocommerce_ppl_shipment pplfilter_a join {$wpdb->prefix}woocommerce_ppl_package pplfilter_b on pplfilter_b.ppl_shipment_id = pplfilter_a.ppl_shipment_id where pplfilter_b.shipment_number like %s )", $args['s'] . "likeMaker"));
            $sql["where"] .= $q;
        }
        return $sql;
    }



    public static function order_detail($data, $order) {
        /**
         * @var \WC_Order $data
         */
        $parcel = self::getParcelshopOrderData($order, true);

        if ($parcel) {
            $data['formatted_parcel_shop'] = "ParcelShop<br/>{$parcel->getName()}<br/>{$parcel->getStreet()}<br/>{$parcel->getZipCode()} {$parcel->getCity()}";
        }
        return $data;

    }

    public static function bulk_action($bulk) {

        /*
        $collections = CollectionData::available_collections();
        foreach($collections as $col) {
            $id = $col->get_id();
            $date = $col->get_reference_id() . ' '. (new \DateTime($col->get_send_date()))->format("d.m.Y");
            $bulk["ppl_bulk_operation_{$id}"] = "Do svozu {$date}";
         }
        */
        $bulk["ppl_bulk_operation_create_labels"] = "Vytisknout zásilky PPL";
        return $bulk;
    }

    public static function handle_bulk_action($ids, $action, $type)
    {
        if (str_contains($action, "ppl_bulk_operation_create_labels") && $type === 'order')
        {
            $shipments = array_reduce(array_map(function ($id) {
                $order = new \WC_Order($id);
                $shipments = ShipmentData::read_order_shipments($id);
                $founded = !!$shipments;
                /**
                 * @var ShipmentData $shipment
                 */
                foreach ($shipments as $key => $shipment) {
                    if ($shipment->get_lock() || $shipment->get_batch_id()) {
                        unset($shipments[$key]);
                    }
                }

                if ($shipments)
                {
                    foreach ($shipments as $k => $v)
                    {
                        $shipments[$k] = Serializer::getInstance()->denormalize($v, ShipmentModel::class);
                    }
                    return $shipments;
                }
                if (!$founded && self::hasPPLShipment($order)) {
                    return [Serializer::getInstance()->denormalize($order, ShipmentModel::class)];
                }
                return [];

            }, $ids), function ($carry, $shipment) {
                return array_merge($carry, $shipment);
            }, []);

            $hasError = false;
            foreach ($shipments as $shipment) {
                $errors  = new Errors();
                Validator::getInstance()->validate($shipment, $errors, "");
                if ($errors->errors) {
                    $hasError = true;
                    set_transient("ppl_bulk_create_label_" . get_current_user_id(), 'Při vytváření zásilek byly nalezeny chyby', 60);
                }

            }
            if (!$hasError) {
                /**
                 * @var ShipmentModel $shipment
                 * @var ShipmentData $shipmentData
                 */
                foreach ($shipments as $key => $shipment) {
                    if (!$shipment->getId()) {
                        $shipmentData = Serializer::getInstance()->denormalize($shipment, ShipmentData::class);
                        $shipmentData->save();
                        $shipments[$key] = $shipmentData->get_id();
                    } else {
                        $shipments[$key] = $shipment->getId();
                    }

                }

                $operation = new CPLOperation();
                try {
                    $operation->createPackages($shipments);
                }
                catch (\Exception $ex) {
                    set_transient("ppl_bulk_create_label_" . get_current_user_id(), "Vznikl problém při vytváření zásilky", 60);
                }
            }


            return $ids;
        }

        return $ids;
    }

    public static function admin_notices() {
        $value = get_transient("ppl_bulk_create_label_" . get_current_user_id());
        if ($value) {
        ?>
        <div class="error notice">
            <p><?php _e( $value, 'woocommerce-ppl-domain' ); ?></p>
        </div>
        <?php
        }
        delete_transient("ppl_bulk_create_label_" . get_current_user_id());
    }

    public static function preview_end() {
?>
        <div id="formatted_parcel_shop" style="display: none">
        </div>
        <# (function()  {
            setTimeout(function() {
                const item = jQuery("#formatted_parcel_shop").closest("article").find(".wc-order-preview-address:nth-child(2)")
                item.append("<br/><br/>" +data.formatted_parcel_shop)
            });
        })()
        #>
<?php
    }

    public static function register() {

        add_action( 'woocommerce_order_list_table_extra_tablenav', [self::class, 'filters'], 10, 2 );
        add_action("woocommerce_admin_order_preview_end", [self::class, "preview_end"], 10, 2);
        add_action("admin_notices", [self::class, "admin_notices"]);

        add_filter( 'woocommerce_order_list_table_prepare_items_query_args', [self::class, 'query_args'] );
        add_filter("woocommerce_admin_order_preview_get_order_details", [self::class, "order_detail"], 10, 2);
        add_filter( 'bulk_actions-woocommerce_page_wc-orders', [self::class, 'bulk_action'], 20, 1 );
       // add_filter('woocommerce_bulk_action_ids', [self::class, 'handle_bulk_action'], 10, 3);

        add_filter( 'woocommerce_orders_table_query_clauses', [self::class, 'query_clausule'], 10, 3);
        add_action("wp_ajax_ppl_filter_table", [self::class, "rerender"]);
            /*
        add_filter( 'bulk_actions-woocommerce_page_wc-orders', function ( $bulk_actions ) {
            $filter = wp_parse_args($_GET, ["ppl_filter" => self::NOTHING]);
            if ($filter["ppl_filter"] === self::REQUIRED) {
                $bulk_actions['ppl_collection'] = 'Vytvořit PPL zásilku';
            }
            return $bulk_actions;
        } );
            */
    }

}