<?php

namespace WoocommercePpl\Admin\Product;

defined("WPINC") or die();

use WoocommercePpl\Admin\Assets\JsTemplate;
use WoocommercePpl\Model\Model\ProductModel;
use WoocommercePpl\Entity\ProductEntity;
use WoocommercePpl\Serializer;
use WoocommercePpl\Model\Model\ShipmentMethodModel;
use WoocommercePpl\ShipmentMethod;

class Tab {

    /**
     * @param array $tabs
     * @return array
     */
    public static function Tabs($tabs)
    {
        $tabs["woocoomerce_ppl_tab"] = [
            "label" => "PPL",
            "target" => "woocommerce_ppl_tab",
            "class" => [ 'hide_if_virtual', 'hide_if_downloadable'  ]
        ];

        return $tabs;
    }

    public static function get_shipping()
    {
        $output = [];
        foreach (ShipmentMethod::methodsWithCod() as $k => $v) {
            $output[] = Serializer::getInstance()->normalize(new ShipmentMethodModel([
                "code" => $k,
                "title" => $v,
                "codAvailable" => ShipmentMethod::isMethodWithCod($k),
                "parcelRequired" => ShipmentMethod::isMethodWithParcel($k)
            ]), "array");
        }
        return $output;
    }

    public static function Render() {
        global $post_id;
        $model = Serializer::getInstance()->denormalize(new \WC_Product($post_id),  ProductModel::class);
        $model = Serializer::getInstance()->normalize($model, "array");
        $shipments = self::get_shipping();

        JsTemplate::scripts();

        ?><div id="woocommerce_ppl_tab" class="panel woocommerce_options_panel">
        </div>
        <script type="text/javascript">
            window.pplPlugin = window.pplPlugin || [];

            pplPlugin.push([
                "pplProductTab",
                "woocommerce_ppl_tab",
                {
                    data: <?php echo json_encode( Serializer::getInstance()->normalize($model, "array") );?>,
                    methods: <?php echo json_encode( $shipments );?>,
                }
            ])
        </script>
        <?php
    }

    public static function Save($post_id) {
        $model = new ProductModel();
        $post = wp_parse_args($_POST, [
                "pplConfirmAge15" => false,
                "pplConfirmAge18"=> false,
                "pplDisabledTransport" => [],
        ]);
        $model->setPplConfirmAge15(!!@$post['pplConfirmAge15']);
        $model->setPplConfirmAge18(!!@$post['pplConfirmAge18']);
        if (is_array($post['pplDisabledTransport'])) {
            $model->setPplDisabledTransport($post['pplDisabledTransport']);
        }
        $product = new \WC_Product($post_id);
        Serializer::getInstance()->denormalize($model, \WC_Product::class, null, [ "product" => $product]);
        $product->save_meta_data();
    }

    public static function register()
    {
        add_filter("woocommerce_product_data_tabs", array(Tab::class, "Tabs"));
        add_action("woocommerce_product_data_panels", array(Tab::class, "Render"));
        add_action("woocommerce_process_product_meta", array(Tab::class, "Save"));

    }
}

