<?php

namespace WoocommercePpl\Front\Components\ParcelShop;

defined("WPINC") or die();

use WoocommercePpl\Model\Model\ParcelDataModel;
use WoocommercePpl\Model\Model\ShipmentCartModel;
use WoocommercePpl\Model\Normalizer\ParcelDataModelNormalizer;
use WoocommercePpl\Serializer;
use WoocommercePpl\Traits\ParcelDataModelTrait;

class BlockOldView
{

    use ParcelDataModelTrait;

    private static $updateOrderReview = false;

    public static function footer()
    {
        wp_enqueue_script("ppl_map_js");
        wp_enqueue_style("ppl_map_css");
        $script = plugins_url("/Assets/select-parcelshop.js", realpath(__DIR__ . '/../'));
        wp_enqueue_script("parcelshop-old-view", $script, [], ppl_get_version());
    }


    public static function woocommerce_checkout_update_order_review($postdata) {

        $shippingMethod = ppl_get_cart_shipping_method();
        if ($shippingMethod) {
            $method = $shippingMethod->get_method_id();

            $methods = isset( $_POST['shipping_method'] ) ? wc_clean( wp_unslash( $_POST['shipping_method'] ) ) : [];
            foreach ($methods as $val) {
                if ($val === $method)
                    return;
            }
            self::$updateOrderReview = true;
        }

    }

    public static function after_shipping($inner = false)
    {

        $shippingMethod = ppl_get_cart_shipping_method();
        if (!$shippingMethod) {
            return;
        }

        if ($shippingMethod->get_method_id() !== $inner->get_method_id())
            return;


        /**
         * @var ShipmentCartModel $cartModel
         */
        $cartModel = Serializer::getInstance()->denormalize($shippingMethod->get_meta_data(), ShipmentCartModel::class);

        if ($cartModel->getParcelRequired() && !is_cart()) {
            $parcelshop = ppl_get_cart_parceldata();

            $ageOk = !$cartModel->getAgeRequired();
            $ageOk = $ageOk || !$parcelshop || $parcelshop && $parcelshop->getAccessPointType() === "ParcelShop";

            wc_get_template("ppl/select-parcelshop-inner.php", [
                "shipping_address" => $parcelshop,
                "cod" => $cartModel->getCost() > 0 ? true: false,
                "parcelRequired" => $cartModel->getParcelRequired(),
                "ageRequired" => $cartModel->getAgeRequired(),
                "mapEnabled" => $cartModel->getParcelRequired(),
                "ageOk" => $ageOk,
                "content" => urlencode(json_encode($parcelshop ? Serializer::getInstance()->normalize($parcelshop) : null)),
                "showMap" => is_ajax() && !$parcelshop && self::$updateOrderReview,
                "image" =>  ppl_asset_icon("ps_pb.png")
            ]);

        }

    }

    public static function full_label($label, \WC_Shipping_Rate $method)
    {

        $id = $method->get_method_id();
        if (str_starts_with($id, create_ppl_name(""))) {
            $metadata = $method->get_meta_data();
            $metadata = Serializer::getInstance()->denormalize($metadata, ShipmentCartModel::class);
            /**
             * @var ShipmentCartModel $metadata
             */
            $img = ppl_asset_icon("small_logo.png");
            $parcelRequired = $metadata->getParcelRequired();
            ob_start();

            wc_get_template("ppl/method-name.php", [
                "img" => $img,
                "label" => $label
            ]);
            return ob_get_clean();

        }
        return $label;
    }

    public static function refresh_payment_method()
    {
        ?>
        <script>
            (function ($) {
                $('form.checkout').on('change', 'input[name^="payment_method"]', function () {
                    $('body').trigger('update_checkout');
                });
            })(jQuery);
        </script>
        <?php
    }

    public static function formatted_address($args, $text)
    {
        /**
         * '{first_name}'       => $args['first_name'],
         * '{last_name}'        => $args['last_name'],
         * '{name}'             => sprintf(
         * '{company}'          => $args['company'],
         * '{address_1}'        => $args['address_1'],
         * '{address_2}'        => $args['address_2'],
         * '{city}'             => $args['city'],
         * '{state}'            => $full_state,
         * '{postcode}'         => $args['postcode'],
         * '{country}'          => $full_country,
         * '{first_name_upper}' => wc_strtoupper($args['first_name']),
         * '{last_name_upper}'  => wc_strtoupper($args['last_name']),
         * '{name_upper}'       => wc_strtoupper(

         * '{company_upper}'    => wc_strtoupper($args['company']),
         * '{address_1_upper}'  => wc_strtoupper($args['address_1']),
         * '{address_2_upper}'  => wc_strtoupper($args['address_2']),
         * '{city_upper}'       => wc_strtoupper($args['city']),
         * '{state_upper}'      => wc_strtoupper($full_state),
         * '{state_code}'       => wc_strtoupper($state),
         * '{postcode_upper}'   => wc_strtoupper($args['postcode']),
         * '{country_upper}'    => wc_strtoupper($full_country),
         */
        $shippingMethod =  ppl_get_cart_shipping_method();
        if (!$shippingMethod)
            return $args;

        /**
         * @var ShipmentCartModel $cartModel
         */
        $cartModel = Serializer::getInstance()->denormalize($shippingMethod->get_meta_data(), ShipmentCartModel::class);
        /**
         * @var ParcelDataModel $parcelshop
         */
        $parcelshop = ppl_get_cart_parceldata();

        if ($cartModel->getParcelRequired()) {
            foreach ($args as $key => $value)
            {
                $args[$key] = '';
            }
        }
        if ($parcelshop)
        {
            $args['{name}'] = "Výdejní místo";
            $args['{company}'] = $parcelshop->getName();
            $args['{address_1}'] = $parcelshop->getStreet();
            $args['{city}'] = $parcelshop->getCity();
            $args['{country}'] = $parcelshop->getCountry();
        }


        return $args;
    }

    public static function label_position()
    {
        wp_enqueue_style("ppl-label-position", plugins_url("Front/Assets/label-method.css", realpath(__DIR__ . '/../../')), [], ppl_get_version());
    }

    public static function register()
    {

        add_action("woocommerce_after_shipping_rate", [self::class, "after_shipping"]);

        if (!is_admin())
        {
            add_filter('woocommerce_formatted_address_replacements', [self::class, "formatted_address"], 10, 2);
        }
        // add_action('woocommerce_review_order_after_shipping', [self::class, "after_shipping"]);

        add_filter("woocommerce_cart_shipping_method_full_label", [self::class, "full_label"], 10, 2);
        add_action("wp_head", [self::class, "label_position"]);
        add_action('woocommerce_review_order_before_payment', [self::class, 'refresh_payment_method']);
        add_action("wp_footer", [self::class, "footer"]);
        add_action("woocommerce_checkout_update_order_review", [self::class, "woocommerce_checkout_update_order_review"]);

    }
}



