<?php
namespace WoocommercePpl;

defined("WPINC") or die();

use WoocommercePpl\Front\Validator\ParcelShopValidator;
use WoocommercePpl\Model\Model\ShipmentCartModel;
use function WpifyWooDeps\DI\create;

class ShipmentMethod extends \WC_Shipping_Method {

    public $parcelBoxRequired = false;

    public $codAvailable = "";

    static $zones_shipments;

    public static function allMethods() {
        return array_merge(array_keys(self::methods()), array_values(self::codMethods()));
    }

    public static function methodsWithCod() {
        $methods = self::methods();
        foreach (self::codMethods() as $k => $v)
        {
            if (isset($methods[$k]))
            {
                $methods[$v] = $methods[$k] . ' - dobírka';
            }
        }
        return $methods;
    }

    public static function isMethodWithCod($code) {
        $cods = array_flip(self::codMethods());
        return isset($cods[$code]);
    }

    public static function isMethodWithParcel($code)
    {
        return in_array($code, self::parcelMethods());
    }

    public static function parcelMethods()
    {
        return [
            "SMAR", "SMEU", "SMAD", "SMED"
        ];
    }

    public static function methods() {
        return [
            "PRIV"=> "PPL Parcel CZ Private", // cz
            "SMAR" => "PPL Parcel CZ Smart", // cz, VM

            "SMEU" => "PPL Parcel Smart Europe", // necz
            "CONN" => "PPL Parcel Connect", // necz
        ];
    }

    public static function methodsDescriptions()
    {
        return [
            "PRIV" => "Doprava v rámci České republiky na adresu",
            "SMAR" => "Doprava v rámci České republiky na výdejní místo",
            "SMEU" => "Doprava v rámci Polska, Německa, Slovenska na výdejní místo",
            "CONN" => "Doprava v rámci EU na adresu"
        ];
    }

    public static function codMethods() {
        return  [
            "BUSS" => "BUSD",
            "DOPO" => "DOPD",
            "PRIV"=> "PRID",
            "CONN" => "COND",
            "SMAR" => "SMAD",
            "SMEU" => "SMED"
        ];

    }

    public function getMethodTitleByCode($paymentCode)
    {
        $cod = @$this->get_instance_option("codPayment");
        if ($cod === $paymentCode) {
            return $this->get_method_title() . " - dobírka";
        }
        return $this->get_method_title();
    }

    public function getMethodCodeByPayment($paymentCode)
    {

        $cod = @$this->get_instance_option("codPayment");
        if ($cod === $paymentCode) {
            $originalCode = str_replace(create_ppl_name(""), "", $this->id);
            return self::codMethods()[$originalCode];
        }
        return str_replace(create_ppl_name(""), "", $this->id);
    }

    public function __construct($method_id)
    {
        parent::__construct(intval($method_id));
        $isInstance = !!intval($method_id);

        if (!self::$zones_shipments || $method_id === (intval($method_id) . "") && !($founded = array_filter(self::$zones_shipments, function ($item) use ($method_id) {
                return $item->instance_id == $method_id;
            }))) {
            global $wpdb;
            $query = $wpdb->prepare("select instance_id, zone_id, method_id from {$wpdb->prefix}woocommerce_shipping_zone_methods where method_id like %s ", create_ppl_name("") . "%");
            $result = $wpdb->get_results($query);
            static::$zones_shipments = $result;
        }

        if ("{$method_id}" === (intval($method_id) . "")) {
            $founded = @$founded ?: array_filter(self::$zones_shipments, function ($item) use ($method_id) {
                return $item->instance_id == $method_id;
            });
            $method_id = reset($founded)->method_id;
            $this->id = $method_id;
            $pplId = str_replace(create_ppl_name(""), "", $method_id);
        } else if ($method_id) {
            $this->id = create_ppl_name($method_id);
            $pplId =  str_replace(create_ppl_name(""), "", $method_id);
        } else
            throw new \Exception();

        $codAvailables = self::codMethods();

        $this->codAvailable = isset($codAvailables[$pplId]) ? @$codAvailables[$pplId] : false;
        $this->parcelBoxRequired = in_array($pplId, self::parcelMethods());

        $this->supports = array(
            "shipping-zones",
            "instance-settings"
        );

        $this->plugin_id = create_ppl_name("");
        $this->title = $this->method_title = self::methods()[$pplId];
        $this->method_description = self::methodsDescriptions()[$pplId];
        /*
        if ($isInstance) {
            $this->init_instance_settings();
        } else {
            $this->init_settings();
        }*/
    }

    public function get_instance_form_fields() {
        if (!$this->instance_form_fields) {
            $this->instance_form_fields = $this->get_form_fields();
            return parent::get_instance_form_fields();
        }
        return parent::get_instance_form_fields();
    }

    public function get_form_fields()
    {
        if (!$this->form_fields) {
            $this->init_form_fields();
            return parent::get_form_fields();
        }
        return parent::get_form_fields();
    }

    public function init_form_fields()
    {
        $form_fields = array(
            'title' => array(
                'title'       => esc_html__('Název dopravy', 'wkwc-shipping' ),
                'type'        => 'text',
                'description' => esc_html__('Enter the method title', 'wkwc-shipping'  ),
                'default'     => esc_html__('', 'wkwc-shipping' ),
                'desc_tip'    => true,
            ),
            'description' => array(
                'title'       => esc_html__('Podrobnějsí popis', 'wkwc-shipping' ),
                'type'        => 'textarea',
                'description' => esc_html__('Enter the Description', 'wkwc-shipping'  ),
                'default'     => esc_html__('', 'wkwc-shipping' ),
                'desc_tip'    => true
            ),
        );

        $zones = \WC_Shipping_Zones::get_zones();
        $currencies  = include __DIR__ . '/config/currencies.php';

        foreach ($zones as $zone) {
            foreach ($zone['shipping_methods'] as $method) {
                if ($method->instance_id == $this->instance_id
                    && $method->id === $this->id) {
                    $codes = array_filter(array_map(function($stdClass) {
                        if ($stdClass->type === "postcode")
                            return null;
                        return $stdClass->code;
                    }, $zone["zone_locations"]));
                    if ($codes) {
                        foreach (array_diff(array_keys($currencies), $codes) as $code) {
                            unset($currencies[$code]);
                        }
                    }
                    break;
                }
            }
        }


        foreach (array_unique(array_merge([ get_option( 'woocommerce_currency' )],array_values($currencies))) as $currency) {
            $form_fields["cost_allow_{$currency}"] = array(
                'title'       => esc_html__("Povolení měny", 'wkwc-shipping' ),
                'type'        => 'checkbox',
                'description' => esc_html__('Add the method cost', 'wkwc-shipping'  ),
                'default'     => esc_html__('', 'wkwc-shipping' ),
                'desc_tip'    => true
            );

            $form_fields["cost_{$currency}"] = array(
                'title'       => esc_html__("Cena za dopravu", 'wkwc-shipping' ),
                'type'        => 'price',
                'description' => esc_html__('Add the method cost', 'wkwc-shipping'  ),
                'default'     => esc_html__('', 'wkwc-shipping' ),
                'desc_tip'    => true
            );

            $form_fields["cost_order_free_{$currency}"] = array(
                'title'       => esc_html__("Od jaké ceny bude doprava zadarmo", 'wkwc-shipping' ),
                'type'        => 'price',
                'description' => esc_html__('Add the method cost', 'wkwc-shipping'  ),
                'default'     => esc_html__('', 'wkwc-shipping' ),
                'desc_tip'    => true
            );

            if (in_array($currency, ['CZK', "EUR", "PLN", "HUF", "RON"])) {
                $form_fields["cost_cod_fee_{$currency}"] = array(
                    'title' => esc_html__("Příplatek za dobírku", 'wkwc-shipping'),
                    'type' => 'price',
                    'description' => esc_html__('Add the method cost', 'wkwc-shipping'),
                    'default' => esc_html__('', 'wkwc-shipping'),
                    'desc_tip' => true
                );

                $form_fields["cost_order_free_cod_{$currency}"] = array(
                    'title' => esc_html__("Od jaké ceny bude doprava zadarmo pro dobírku", 'wkwc-shipping'),
                    'type' => 'price',
                    'description' => esc_html__('Add the method cost', 'wkwc-shipping'),
                    'default' => esc_html__('', 'wkwc-shipping'),
                    'desc_tip' => true
                );
            }
        }

        $payments = WC()->payment_gateways()->payment_gateways();
        foreach ($payments as $key => $value )
        {
            $payments[$key] = $value->get_title();
        }

        $payments[""] = "Nevybráno";

        $form_fields["disablePayments"] = array(
            'title'       => esc_html__('Platby, které nejsou povoleny', 'wkwc-shipping' ),
            'type'        => 'multiselect',
            'description' => esc_html__(!$this->codAvailable ? 'Tato metoda nepodoruje dobírku, prosim označte tuto platbu, pokud existuje' : "", 'wkwc-shipping'  ),
            "options" => $payments,
            'default'     => [],
            'desc_tip'    => true
        );

        $form_fields["codPayment"] = [
            'title'       => esc_html__('Platba považovaná za dobírku', 'wkwc-shipping' ),
            'type'        => 'select',
            'description' => esc_html__("Vyberte platbu určenou na dobírku (u teto platby je nutno řešit i nastaveni v rámci pokladny, pro jeji zobrazení)", 'wkwc-shipping'  ),
            "options" =>  $payments,
            'default'     => "",
            'desc_tip'    => true
        ];



        $this->form_fields = $form_fields;
    }

    public static function shipping_methods($shippings)
    {

        return array_merge($shippings, [
            create_ppl_name("PRIV" )=> new ShipmentMethod("PRIV"),
            create_ppl_name("SMAR") =>  new ShipmentMethod("SMAR"),
            create_ppl_name("SMEU")=>  new ShipmentMethod("SMEU"),
            create_ppl_name("CONN") => new ShipmentMethod("CONN"),
        ]);
    }


    /**
     * Výpočet ceny dopravy
     */
    public function calculate_shipping($package = array())
    {
        $enabled = $this->enabled;

        if ($enabled === "no")
            return;

        $cart = wc()->cart;

        if (!$cart) {
            wc()->initialize_cart();
            $cart = wc()->cart;
        }

        $curentCurrency = get_woocommerce_currency();

        if (@$this->get_instance_option("cost_allow_{$curentCurrency}") !== 'yes'){
            return;
        }

        $countries = include __DIR__ . '/config/countries.php';


        $country = WC()->cart->get_customer()->get_shipping_country('');

        if (!isset($countries[$country]))
            return;

        /**
         * @var ShipmentCartModel $cartData
         */
        $cartData = Serializer::getInstance()->denormalize($this, ShipmentCartModel::class);

        if ($cartData->getParcelRequired() && $cartData->getAgeRequired() && $country !== 'CZ')
            return;

        if ($cartData->getDisabledByProduct())
            return;


        $this->add_rate([
            'id' => $this->id,
            'label' => $this->instance_settings["title"] ?: $this->title ?: $this->method_title,
            'cost' => $cartData->getCost(),
            'calc_tax' => 'per_order',
            "meta_data" => Serializer::getInstance()->normalize($cartData, 'array')
        ]);

    }

    public static function recalculate_fees($cart)
    {
        $rate = ppl_get_cart_shipping_method();
        if (!$rate)
            return $cart;

        $metadata = Serializer::getInstance()->denormalize(new ShipmentMethod($rate->get_instance_id() ?: $rate->get_method_id()), ShipmentCartModel::class);
        if ($metadata->isInitialized('codFee') && $metadata->getCodFee()) {
            WC()->cart->add_fee("Příplatek za dobírku", $metadata->getCodFee());
        }
        return $cart;

    }

    public static function available_payment_methods($available_gateways) {
        if ( is_admin() ) return $available_gateways;

        // Získání aktuálně zvoleného způsobu dopravy
        $session = WC()->session;
        if (!$session)
            return $available_gateways;

        $rate = ppl_get_cart_shipping_method();
        if (!$rate)
        {
            return $available_gateways;
        }

        /**
         * @var ShipmentCartModel $metadata
         */
        $metadata = Serializer::getInstance()->denormalize($rate->get_meta_data(), ShipmentCartModel::class);
        if ($metadata->isInitialized("disablePayments") && $metadata->getDisablePayments()) {
            $disablePayments = $metadata->getDisablePayments();
            foreach ( $available_gateways as $gateway_id => $gateway ) {
                if ( in_array( $gateway_id, $disablePayments ) ) {
                    unset( $available_gateways[$gateway_id] );
                }
            }
        }

        if ($metadata->getDisableCod() && $metadata->getCodPayment()) {
            $cod = $metadata->getCodPayment();
            unset($available_gateways[$cod]);
        }

        return $available_gateways;

    }


    public function generate_settings_html( $form_fields = array(), $echo = true ) {

        $html = parent::generate_settings_html($form_fields, $echo);
        ob_start();
        $id = create_ppl_name($this->id . '_description');
        $lastId = create_ppl_name($this->id . '_disablePayments');
        $titleId = create_ppl_name($this->id . '_about');
        $currencies = array_reduce(array_filter(array_map(function ($item) {
            if (preg_match('~_([A-Z]{3})$~', $item, $match)) {
                return create_ppl_name($this->id . '_' .$item);
            }
            return null;
        }, array_keys($form_fields))), function ($acc, $item) {
            preg_match('~_([A-Z]{3})$~', $item, $match);
            $acc[$match[1]][] = $item;
            return $acc;
        }, []);
        $allInputs = array_merge(...array_values($currencies));

        ?>
        <script type="text/javascript">
            (function() {
                var currencies = <?php echo json_encode(array_keys($currencies)); ?>;
                jQuery('#<?php echo $lastId ?>').closest('tr').before(`<tr><td colspan="50">
 <hr>
</td></tr>`);
                var data = jQuery(`<tr>
      <th colspan="50" class="titledesc">
          ${currencies.map(x => {
                    return `<a href='#currency_${x}'>${x}</a>`;
                }).join(" | ")}
        <hr>
<div id="<?php echo $titleId ?>"></div>
      </th>
</tr>`).on('click', 'a', function (ev) {
                    ev.preventDefault();
                    jQuery(ev.currentTarget).closest('tr').find('a').css('font-weight', 'normal');
                    jQuery(ev.currentTarget).css('font-weight', 'bold');
                    const currency = `${ev.currentTarget.href}`.match(/currency_([A-Z]{3})$/)[1];
                    <?php echo json_encode($allInputs) ?>.forEach(function(x){
                        var cur = x.substring(x.length-3);
                        if (cur === currency)
                            jQuery('#' + x).closest('tr').show();
                        else
                            jQuery('#' + x).closest('tr').hide();
                    })
                    jQuery('#<?php echo $titleId ?>').html(`Nastavení pro měnu: ${currency}`)

                });
                jQuery("#<?php echo $id?>").closest('tr').after(data);
                data.find('a:first-child').click();
            })();
        </script>
        <?php
        if ($echo)
            echo ob_get_clean();
        else
            return $html . ob_get_clean();

    }

    public static function hide_order_itemmeta($metas)
    {
        return array_merge(['disablePayments', "parcelRequired", "mapEnabled", "codFee", "codPayment", "disableCod", "ageRequired", "cost"], $metas );
    }

    public static function register() {
        add_filter("woocommerce_shipping_methods", [self::class, 'shipping_methods']);
        add_filter('woocommerce_hidden_order_itemmeta', [self::class, 'hide_order_itemmeta']);
        add_filter("woocommerce_available_payment_gateways", [self::class, "available_payment_methods"]);
        add_filter('woocommerce_cart_calculate_fees', [self::class, 'recalculate_fees']);
    }
}