<?php
namespace WoocommercePpl\Validator;
defined("WPINC") or die();

use WoocommercePpl\Model\Model\BankAccountModel;
use WoocommercePpl\Model\Model\ShipmentModel;
use WoocommercePpl\Model\Model\UpdateShipmentModel;

abstract class ModelValidator {
    /**
     * @param mixed $model
     * @return bool
     */
    public abstract function canValidate($model);

    /**
     * @param mixed $model
     * @param \WP_Error $errors
     * @param string $path
     * @return void
     */
    public abstract function validate($model, $errors, $path);

    public function getValue($model, $property)
    {
        /**
         * @var UpdateShipmentModel $modelPath
         */
        foreach (explode(".", $property) as $modelPath) {
            if(is_callable([$model,"isInitialized"]))
            {
                if ($model->isInitialized($modelPath)) {
                    $model = $model->{"get{$modelPath}"}();
                    if (!$model)
                        return null;
                }
                else {
                    return null;
                }
            }
            else if (is_array($model))
            {
                if (isset($model[$modelPath]))
                    $model = $model[$modelPath];
                else
                    return null;
            }
            else {
                throw new \Exception("Cesta vs model");
            }
        }
        return $model;
    }


    protected static $validators = [
        AddressModelValidator::class,
        CartValidator::class,
        PackageValidator::class,
        ShipmentValidator::class,
        CollectionValidator::class,
        CodValidator::class,
        InsuranceValidator::class,
        ParcelValidator::class
    ];

    public function isCurrency($value)
    {
        return isset(get_woocommerce_currencies()[$value]);
    }


    public function isPhone($phoneNumber)
    {
        $phoneNumber = preg_replace('/[\s\-()]/', '', $phoneNumber);

        // Použít regulární výraz pro validaci formátu telefonního čísla
        $pattern = '/^\+?[0-9]{9,14}$/';

        if (preg_match($pattern, $phoneNumber)) {
            return true;
        } else {
            return false;
        }
    }
}