<?php

use WoocommercePpl\Model\Model\ParcelDataModel;
use WoocommercePpl\Model\Model\ShipmentCartModel;
use WoocommercePpl\Serializer;

defined("WPINC") or die();

function create_ppl_name($name) {
    return "woocommerce_ppl_" . $name;
}

function ppl_map_args() {
    return WoocommercePpl\Front\Components\Map\Map::args();
}

function ppl_asset_icon($name) {

    return plugins_url("src/Admin/Assets/Images/$name", realpath(__DIR__));
}

function ppl_get_allowed_countries() {
    $countries_obj = new \WC_Countries();

    $get_countries = $countries_obj->get_allowed_countries();
    $output = [];

    $countries = include __DIR__ . '/config/countries.php';
    foreach ($get_countries as $key => $v) {
        if (!isset($countries[$key]))
            unset($get_countries[$key]);
    }

    return $get_countries;
}

function ppl_get_cod_currencies() {
    $currencies = include __DIR__ . '/config/cod_currencies.php';
    return $currencies;
}

function ppl_set_phase_max_sync($value)
{
    add_option(create_ppl_name("watch_phases_max_sync"), intval($value) ?: 200) || update_option(create_ppl_name("watch_phases_max_sync"), intval($value) ?: 200);
}

function get_ppl_phase_max_sync() {
    $value = get_option(create_ppl_name("watch_phases_max_sync"));
    return intval($value) ?: 200;
}

function ppl_set_phase($key, $watch) {
    if ($watch)
        add_option(create_ppl_name("watch_phases_{$key}"), true) || update_option(create_ppl_name("_watch_phases_{$key}"), true);
    else
        delete_option(create_ppl_name("watch_phases_{$key}"));
}

function ppl_get_phases() {
    $phases = include  __DIR__ . '/config/shipment_phases.php';
    return array_map(function ($item, $key)  {
        $output = [
            'code' => $key,
            'title' => $item,
            'watch' =>  !!get_option(create_ppl_name("watch_phases_{$key}"))
        ];

        return $output;
    }, $phases, array_keys($phases));

}

function ppl_get_version() {
    if( !function_exists('get_plugin_data') ){
        require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
    }
    $pluginData = get_plugin_data( __DIR__ . '/../index.php' );
    return $pluginData['Version'];
}

/**
 * @return ParcelDataModel|null
 */
function ppl_get_cart_parceldata() {
    $rate = ppl_get_cart_shipping_method();
    if (!$rate)
        return null;

    /**
     * @var ShipmentCartModel $metadata
     */
    $metadata = Serializer::getInstance()->denormalize($rate->get_meta_data(), ShipmentCartModel::class);
    if ($metadata->getParcelRequired()) {
        $session = wc()->session;
        $parcelshop_data = $session->get("ppl_parcelshop_data");
        if ($parcelshop_data) {
            return Serializer::getInstance()->denormalize($parcelshop_data, ParcelDataModel::class);
        }
    }
    return null;
}

function ppl_set_cart_parceldata(?ParcelDataModel $data)
{
    $session = wc()->session;
    if ($data)
        $session->set("ppl_parcelshop_data", Serializer::getInstance()->normalize($data));
    else
        $session->set("ppl_parcelshop_data", null);
}

/**
 * @return \WC_Shipping_Rate|null
 */
function ppl_get_cart_shipping_method()
{
   $session = WC()->session;
    if (!$session)
        WC()->initialize_session();
    $session = WC()->session;
    $cart = WC()->cart;
    if (!$cart)
        WC()->initialize_cart();
    $cart = WC()->cart;

    $chosen_shipping_methods = $session->get( 'chosen_shipping_methods' );
    if (!$chosen_shipping_methods)
        return null;


    $chosen_shipping_method  = $chosen_shipping_methods[0];

    foreach ($chosen_shipping_methods as $key => $shipping_method) {
        $method = str_replace(create_ppl_name(""), "", $chosen_shipping_method);
        $methods = \WoocommercePpl\ShipmentMethod::methods();
        if (isset($methods[$method])) {
            $shipping = WC()->session->get("shipping_for_package_{$key}");
            /**
             * @var \WC_Shipping_Rate $rate
             */
            $rate = @$shipping["rates"][$chosen_shipping_method];
            if ($rate)
                return $rate;
            // problem se zasilkovnou, proste to bez diskuzi vycisti
            $cart->calculate_shipping();
            $shipping = WC()->session->get("shipping_for_package_{$key}");
            $rate = $shipping["rates"][$chosen_shipping_method];
            return $rate;
        }
    }
    return null;
}

function woocommerce_ppl_tables ($activate = false) {
    $version = get_option(create_ppl_name("version"));
    if ($version !== ppl_get_version()) {


        global $wpdb;
        $table = $wpdb->prefix . 'woocommerce_ppl_address';
        $charset = $wpdb->get_charset_collate();
        $charset_collate = $wpdb->get_charset_collate();
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        $sql = "CREATE TABLE `$table` (
  `ppl_address_id` bigint(20) NOT NULL AUTO_INCREMENT,
  `address_name` varchar(40) DEFAULT NULL,
  `name` varchar(100) DEFAULT NULL,
  `contact` varchar(100) DEFAULT NULL,
  `mail` varchar(100) DEFAULT NULL,
  `phone` varchar(50) DEFAULT NULL,
  `street` varchar(50) DEFAULT NULL,
  `city` varchar(50) DEFAULT NULL,
  `zip` varchar(10) DEFAULT NULL,
  `country` varchar(2) DEFAULT NULL,
  `type` varchar(10) NOT NULL,
  `note` varchar(300) DEFAULT NULL,
  `draft` datetime DEFAULT NULL,
  `hidden` tinyint(4) NOT NULL,
  `lock` tinyint(4) NOT NULL,
  PRIMARY KEY (`ppl_address_id`)
) $charset_collate";
        dbDelta($sql);

        $wpdb->query("ALTER TABLE `{$wpdb->prefix}woocommerce_ppl_address` CHANGE `name` `name` varchar(100) COLLATE 'utf8mb4_general_ci' NULL AFTER `address_name`");
        $wpdb->query("
ALTER TABLE `{$wpdb->prefix}woocommerce_ppl_address`
CHANGE `street` `street` varchar(50) COLLATE 'utf8mb4_general_ci' NULL AFTER `phone`,
CHANGE `city` `city` varchar(50) COLLATE 'utf8mb4_general_ci' NULL AFTER `street`,
CHANGE `zip` `zip` varchar(10) COLLATE 'utf8mb4_general_ci' NULL AFTER `city`,
CHANGE `country` `country` varchar(2) COLLATE 'utf8mb4_general_ci' NULL AFTER `zip`;
        ");

        $table = $wpdb->prefix . 'woocommerce_ppl_cod_bank_account';

        $sql = "CREATE TABLE `$table` (
    `ppl_cod_bank_account_id` int(11) NOT NULL AUTO_INCREMENT,
  `account_name` varchar(50) NOT NULL,
  `account` varchar(20) DEFAULT NULL,
  `account_prefix` varchar(20) DEFAULT NULL,
  `bank_code` varchar(7) DEFAULT NULL,
  `iban` varchar(20) DEFAULT NULL,
  `swift` varchar(20) DEFAULT NULL,
  `currency` varchar(3) NOT NULL,
  `draft` timestamp NULL DEFAULT current_timestamp(),
  `lock` tinyint(4) NOT NULL,
  PRIMARY KEY (`ppl_cod_bank_account_id`)
) $charset_collate";

        dbDelta($sql);

        $table = $wpdb->prefix . 'woocommerce_ppl_collections';
        $sql = "CREATE TABLE `$table` (
    `ppl_collection_id` bigint(20) NOT NULL AUTO_INCREMENT,
  `remote_collection_id` varchar(80) DEFAULT NULL,
  `created_date` datetime NOT NULL,
  `send_date` datetime NOT NULL,
  `send_to_api_date` datetime DEFAULT NULL,
  `reference_id` varchar(50) NOT NULL,
  `state` varchar(20) DEFAULT NULL,
  `shipment_count` int(11) NOT NULL,
  `estimated_shipment_count` int(11) NOT NULL,
  `contact` varchar(255) DEFAULT NULL,
  `email` varchar(255) DEFAULT NULL,
  `telephone` varchar(30) DEFAULT NULL,
  `note` varchar(200) DEFAULT NULL,
  PRIMARY KEY (`ppl_collection_id`),
  UNIQUE KEY `reference_id` (`reference_id`)
) $charset_collate";
        dbDelta($sql);

        $table = $wpdb->prefix . 'woocommerce_ppl_package';
        $sql = "CREATE TABLE `$table` (
    `ppl_package_id` bigint(20) NOT NULL AUTO_INCREMENT,
  `ppl_shipment_id` bigint(20) DEFAULT NULL,
  `reference_id` varchar(40) DEFAULT NULL,
  `wc_order_id` bigint(20) DEFAULT NULL,
  `phase` varchar(20) NOT NULL,
  `status` int DEFAULT NULL,
  `status_label` varchar(80) DEFAULT NULL,
  `phase_label` varchar(80) DEFAULT NULL,
  `last_update_phase` datetime DEFAULT NULL,
  `last_test_phase` datetime DEFAULT NULL,
  `ignore_phase` tinyint(4) DEFAULT NULL,
  `shipment_number` varchar(40) DEFAULT NULL,
  `weight` decimal(10,0) DEFAULT NULL,
  `insurance` decimal(10,0) DEFAULT NULL,
  `insurance_currency` varchar(3) DEFAULT NULL,
  `import_error` text DEFAULT NULL,
  `import_error_code` text DEFAULT NULL,
  `label_id` text DEFAULT NULL,
  `draft` timestamp NULL DEFAULT current_timestamp(),
  `lock` tinyint(4) NOT NULL,
  PRIMARY KEY (`ppl_package_id`)
) $charset_collate";
        dbDelta($sql);

        $table = $wpdb->prefix . 'woocommerce_ppl_parcel';

        $sql = "CREATE TABLE `$table` (
    `ppl_parcel_id` bigint(20) NOT NULL AUTO_INCREMENT,
  `type` varchar(20) NOT NULL,
  `name` varchar(100) NOT NULL,
  `name2` varchar(100) DEFAULT NULL,
  `street` varchar(50) NOT NULL,
  `city` varchar(50) NOT NULL,
  `zip` varchar(10) NOT NULL,
  `code` varchar(20) NOT NULL,
  `country` varchar(2) NOT NULL,
  `draft` timestamp NULL DEFAULT current_timestamp(),
  `lat` double NOT NULL,
  `lng` double NOT NULL,
  `valid` tinyint(4) NOT NULL,
  `hidden` tinyint(4) NOT NULL,
  `lock` tinyint(4) NOT NULL,
  PRIMARY KEY (`ppl_parcel_id`),
  UNIQUE KEY `code` (`code`)
) $charset_collate";

        dbDelta($sql);

        $table = $wpdb->prefix . 'woocommerce_ppl_shipment';

        $sql = "CREATE TABLE `$table` (
    `ppl_shipment_id` int(11) NOT NULL AUTO_INCREMENT,
  `wc_order_id` int(11) DEFAULT NULL,
  `import_errors` text DEFAULT NULL,
  `reference_id` varchar(50) NOT NULL,
  `package_ids` text DEFAULT NULL,
  `import_state` varchar(20) NOT NULL,
  `service_code` varchar(20) DEFAULT NULL,
  `service_name` varchar(40) DEFAULT NULL,
  `recipient_address_id` int(11) DEFAULT NULL,
  `sender_address_id` int(11) DEFAULT NULL,
  `return_address_id` int(11) DEFAULT NULL,
  `cod_value` decimal(10,0) DEFAULT NULL,
  `cod_value_currency` varchar(4) DEFAULT NULL,
  `cod_variable_number` varchar(10) DEFAULT NULL,
  `cod_bank_account_id` int(11) DEFAULT NULL,
  `has_parcel` tinyint(4) NOT NULL,
  `parcel_id` int(11) DEFAULT NULL,
  `batch_id` varchar(50) DEFAULT NULL,
  `batch_label_group` datetime DEFAULT NULL,
  `note` varchar(300) DEFAULT NULL,
  `age` varchar(3) DEFAULT NULL,
  `lock` tinyint(4) NOT NULL,
  `draft` timestamp NULL DEFAULT current_timestamp(),
  PRIMARY KEY (`ppl_shipment_id`),
  UNIQUE KEY `reference_id` (`reference_id`)
) $charset_collate";

        dbDelta($sql);
        add_action("action_scheduler_init", function() use ($activate) {
            as_unschedule_action("woocommerceppl_refresh_shipments_cron");
            as_unschedule_action("woocommerceppl_refresh_setting_cron");
            as_schedule_recurring_action(time(), 60 * 120, "woocommerceppl_refresh_shipments_cron");
            as_schedule_recurring_action(time(), 60 * 60 * 24, "woocommerceppl_refresh_setting_cron");
            if (!$activate)
                add_option(create_ppl_name("version"), ppl_get_version()) || update_option(create_ppl_name("version"), ppl_get_version());
        });
    }


    $rules = get_option(create_ppl_name("rules_version"));

    if ($rules !== ppl_get_version() || $activate)
    {
        add_action("wp_loaded", function() use ($activate) {
            flush_rewrite_rules();
            if (!$activate) {
                add_option(create_ppl_name("rules_version"), ppl_get_version()) || update_option(create_ppl_name("rules_version"), ppl_get_version());
            }
        });
    }

    if ($activate)
    {
        delete_option(create_ppl_name("rules_version"));
        delete_option(create_ppl_name("version"));
    }
}

function woocommerce_ppl_activate () {
    woocommerce_ppl_tables(true);
}

function woocommerce_ppl_deactivate()
{
    as_unschedule_action("woocommerceppl_refresh_shipments_cron");
    as_unschedule_action("woocommerceppl_refresh_setting_cron");
}
